/*
 * Sicherheitsunterweisung
 * Copyright (C) 2023  Jason Streifling
 *
 * This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package session

import (
	"fmt"
	"log"
	"net/http"
	"strings"

	"streifling.com/jason/sicherheitsunterweisung/packages/data"
)

type Mux struct {
	*http.ServeMux
}

func NewMux() *Mux {
	return &Mux{ServeMux: http.NewServeMux()}
}

func (mux *Mux) handleParticipants(db *data.DB, cp <-chan *Participant, b *Briefing) {
	for p := range cp {
		p.GivenAnswers = make([]int, len(b.Questions))

		mux.HandleFunc("/submit-participant/"+fmt.Sprint(b.UUID)+"/"+fmt.Sprint(p.Login)+"/", p.HandleParticipant(db, b))
		mux.HandleFunc("/end-video/"+fmt.Sprint(b.UUID)+"/"+fmt.Sprint(p.Login)+"/", p.HandleEndOfVideo(b))
		var i int
		for i = range b.Questions {
			mux.HandleFunc("/submit-answer/"+fmt.Sprint(b.UUID)+"/"+fmt.Sprint(p.Login)+"/"+fmt.Sprint(i+1)+"/", p.HandleAnswer(db, b, int64(i+1)))
		}
		mux.HandleFunc("/allow-retry/"+fmt.Sprint(b.UUID)+"/"+fmt.Sprint(p.Login)+"/", p.HandleAllowRetry())
		mux.HandleFunc("/retry/"+fmt.Sprint(b.UUID)+"/"+fmt.Sprint(p.Login)+"/", p.HandleRetry(b, &i))
		mux.HandleFunc("/refresh-summary/"+fmt.Sprint(b.UUID)+"/"+fmt.Sprint(p.Login)+"/", p.HandleRefresh(b))
	}
}

func getQuestions(db *data.DB, b *Briefing) {
	questionIDs := make([]string, 4)

	for i := 0; i < len(questionIDs); i++ {
		questionIDs[i] = fmt.Sprint(i + 1)
	}

	var err error
	b.Questions, err = db.GetQuestions(questionIDs)
	if err != nil {
		log.Fatalln(err)
	}

	for i := range b.Questions {
		for j := range b.Questions[i].Answers {
			if parts := strings.Split(b.Questions[i].Answers[j].Text, ":/"); parts[0] == "file" {
				b.Questions[i].Answers[j].Text = parts[1]
				b.Questions[i].Answers[j].IsImage = true
			} else {
				b.Questions[i].Answers[j].IsImage = false
			}
		}
	}
}

func (mux *Mux) handleBriefings(db *data.DB, cb <-chan *Briefing, s *Session) {
	participantChan := make(chan *Participant)
	for b := range cb {
		getQuestions(db, b)

		mux.HandleFunc("/new-participant/"+fmt.Sprint(b.UUID)+"/", b.HandleNewParticipant(participantChan))
		mux.HandleFunc("/submit-form/"+fmt.Sprint(b.UUID)+"/", b.HandleBriefingForm(db, s))

		go mux.handleParticipants(db, participantChan, b)
	}
}

func (mux *Mux) HandleSessions(db *data.DB, cs <-chan *Session, ss *[]*Session) {
	briefingChan := make(chan *Briefing)
	for s := range cs {
		(*ss) = append((*ss), s)

		mux.HandleFunc("/search/"+fmt.Sprint(s.UUID)+"/", s.HandleSearch(db))
		mux.HandleFunc("/new-briefing/"+fmt.Sprint(s.UUID)+"/", s.HandleNewBriefing(briefingChan))
		mux.HandleFunc("/briefing-done/"+fmt.Sprint(s.UUID)+"/", s.HandleBriefingDone(db))

		go mux.handleBriefings(db, briefingChan, s)
	}
}
